@@ -14,19 +14,40 @@ or in python: import sys; print (sys.version)
 inspired by gui2py, easygui
 =============================================
 Install:
-Requires wxPython 2.9.2.4 (tested under canopy)
-You have to install it first manually from
-http://sourceforge.net/projects/wxpython/files/wxPython/2.9.2.4/
-Then
-https://pypi.python.org/pypi/quickgui
+1) Requires wxPython 2.9.2.4 (tested under canopy)
+2) or you can install it first manually from
+    http://sourceforge.net/projects/wxpython/files/wxPython/2.9.2.4/
+Then https://pypi.python.org/pypi/quickgui
 pip install quickgui
-
-For anacoda python (wxPython 3.0 tested)
+method 1&2 run: python myscript.py is fine
+3) For anacoda python (wxPython 3.0 tested)
 but remember to run: pythonw myscript.py instead of python myscript.py
 
 Usage:
 import quickgui as q
 
+Message(msg, seconds=10), message
+    Displays a timed modal message box, timeout and cancel returns 0, ok returns 1
+XPrinter()
+    Display a window to capture print output
+    if on, both terminal and window (updating gui will greatly increase script execution time)
+    if off, only terminal
+
+    Methods: on/off
+
+    Examples:
+        xprinter = XPrinter()
+        xprinter.on()
+        print 'will be shown on window'
+        xprinter.off()
+        print 'will be shown in terminal'
+        xprinter.on()
+        print 'on window again'
+
+        for x in range(100):
+            print "I am a line of " + str(x)
+            # time.sleep(0.01)   
+            
 alert, confirm, getfile, setfile, getdir, inputs
 Alert(message, title="", icon="exclamation")
     # Shows a simple pop-up modal dialog.
@@ -47,7 +68,7 @@ SetFile(directory='', filename='', overwrite=False, wildcard='All Files (*.*)|*.
 GetDir(path="", title='Select a directory')
     # Shows a dialog to choose a directory.
 
-values = Inputs(items=[], instruction='Click the button to read the help.', title='Ask for inputs')
+values = Inputs(items=[], width=None, instruction='Click the button to read the help.', title='Ask for inputs')
     # Flexible dialog for user inputs.
     # Returns a value for each row in a list, e.g. [u'1001', u'female', u'', []]
         # textbox accepts a string or str(default) and returns a string or eval(string)
@@ -94,9 +115,24 @@ values = Inputs(items=[], instruction='Click the button to read the help.', titl
 """
 
 
-import wx
+import wx, sys
 from wx.lib import dialogs
 
+# http://wxpython-users.1045709.n5.nabble.com/Close-Exit-and-Destroy-td2330883.html
+# close(), destroy()
+# You use Close() when you want to programatically tell the frame to go
+# close itself, and is functionally the same as the user telling it to
+# close itself with the "X" button.
+
+# Destroy() tells wx to delete the C++ object instance that corresponds to
+# the frame.  Normally it will destroy itself when it closes in the
+# default EVT_CLOSE event handler, but if you catch the EVT_CLOSE yourself
+# you either need to call Destroy in your handler, or call event.Skip so
+# the default handler will still run.  The EVT_CLOSE handler is where you
+# would normally put the code that checks for open files, asks the user if
+# she wants to save them or cancel, etc.  Based on the user's response you
+# can veto the close if you want. 
+
 def Alert(message, title="", icon="exclamation", scrolled=False, parent=None):
     """
     Alert(message, title="", icon="exclamation")
@@ -239,13 +275,16 @@ class _Inputs(wx.Dialog):
     values = Inputs(items=items)    # returns a list of inputs in the order displayed on the GUI (the insertion of blank line, i.e. ('') in the above example, does not interfere the order of returned values)
     """
 
-    def __init__(self, items=[], instruction='', title=''):
+    def __init__(self, items=[], width=None, instruction='', title=''):
         instruction = str(instruction)
         wx.Dialog.__init__(self, None, -1, title=title, style=wx.DEFAULT_FRAME_STYLE)
 
         szrMain = wx.BoxSizer(wx.VERTICAL)  # default boxsizer
-        szrSub = wx.FlexGridSizer(len(items)+1, 2, 15, 10)   # flexible gridsizer, +1 for help/cancel/ok buttons
-
+        if width:
+            szrSub = wx.FlexGridSizer(len(items)+1+1, 2, 15, 10)   # flexible gridsizer, +1 for help/cancel/ok buttons
+        else:
+            szrSub = wx.FlexGridSizer(len(items)+1, 2, 15, 10)
+            
         # parse items
         self.szrMain = szrMain
         self.widgets = []
@@ -318,6 +357,19 @@ class _Inputs(wx.Dialog):
                 szrSub.Add(label, 0, wx.ALL | wx.EXPAND | wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL, 0)
                 label = wx.StaticText(self, wx.ID_ANY, '')
                 szrSub.Add(label, 0, wx.ALL | wx.EXPAND | wx.ALIGN_CENTER_VERTICAL, 0)
+        
+        if width:
+            # label = wx.StaticText(self, wx.ID_ANY, '', style=wx.ALIGN_RIGHT)
+            # szrSub.Add(label, 0, wx.ALL | wx.EXPAND | wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL, 0)
+            # textbox = wx.TextCtrl(self, wx.ID_ANY, ' '*width, size=(len(' '*width*10),-1))
+            # color = wx.SystemSettings.GetColour(wx.SYS_COLOUR_BACKGROUND)
+            # textbox.SetBackgroundColour(color)
+            # textbox.Enable(False)
+            # szrSub.Add(textbox, 0, wx.ALL | wx.EXPAND | wx.ALIGN_CENTER_VERTICAL, 0)
+            label = wx.StaticText(self, wx.ID_ANY, '', style=wx.ALIGN_RIGHT)
+            szrSub.Add(label, 0, wx.ALL | wx.EXPAND | wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL, 0)
+            label = wx.StaticText(self, wx.ID_ANY, ' '*width)
+            szrSub.Add(label, 0, wx.ALL | wx.EXPAND | wx.ALIGN_CENTER_VERTICAL, 0)
 
         btnHelp = wx.Button(self, wx.ID_HELP, "Help")
         ## msecs; Set the delay after which the tooltip disappears or how long a tooltip remains visible.
@@ -377,13 +429,15 @@ class _Inputs(wx.Dialog):
         return values
 
 
-def Inputs(items=[], instruction='Click the button to read the help.', title='Ask for inputs'):
+def Inputs(items=[], width=None, instruction='Click the button to read the help.', title='Ask for inputs'):
     """Flexible dialog for user inputs."""
-    dlg = _Inputs(items=items, instruction=instruction, title=title)
+    dlg = _Inputs(items=items, width=width, instruction=instruction, title=title)
     
     # communicate between dlg and this function
     # the dlg should also have a cancel button; if only has OK, then click the x close button returns wx.ID_OK
-    if dlg.ShowModal() == wx.ID_OK:
+    answer = dlg.ShowModal()
+    dlg.ToggleWindowStyle(wxSTAY_ON_TOP)
+    if answer == wx.ID_OK:
         values = dlg.return_values()
     else:
         values = None
@@ -392,26 +446,202 @@ def Inputs(items=[], instruction='Click the button to read the help.', title='As
     return values
 inputs = Inputs
 
+
+class _TimedMessageDialog(wx.Dialog):
+    def __init__(self, message, title, ttl=10):
+        wx.Dialog.__init__(self, None, -1, title,size=(400, 150))
+        self.CenterOnScreen(wx.BOTH)
+        self.timeToLive = ttl
+
+        stdBtnSizer = self.CreateStdDialogButtonSizer(wx.OK|wx.CANCEL) 
+        stMsg = wx.StaticText(self, -1, message)
+        self.stTTLmsg = wx.StaticText(self, -1, 'Closing this dialog box in %d s...'%self.timeToLive)
+
+        vbox = wx.BoxSizer(wx.VERTICAL)
+        vbox.Add(stMsg, 1, wx.ALIGN_CENTER|wx.TOP, 10)
+        vbox.Add(self.stTTLmsg,1, wx.ALIGN_CENTER|wx.TOP, 10)
+        vbox.Add(stdBtnSizer,1, wx.ALIGN_CENTER|wx.TOP, 10)
+        self.SetSizer(vbox)
+
+        self.timer = wx.Timer(self)
+        self.timer.Start(1000) #Generate a timer event every second
+        self.Bind(wx.EVT_TIMER, self.onTimer, self.timer)
+
+    def onTimer(self, evt):
+        self.timeToLive -= 1
+        self.stTTLmsg.SetLabel('Close itself in %d s...'%self.timeToLive)
+
+        if self.timeToLive <= 0:
+            self.timer.Stop()
+            self.Close()
+
+def Message(msg, seconds=10):
+    """Message(msg, seconds=10)
+    Displays a timed modal message box
+    timeout and cancel returns 0, ok returns 1
+    """
+    dlg = _TimedMessageDialog(msg, 'Message', seconds)               
+    result = dlg.ShowModal()
+    if result == wx.ID_OK:
+        return 1
+    else:
+        return 0
+message = Message
+
+#####################################################################
+# http://www.blog.pythonlibrary.org/2009/01/01/wxpython-redirecting-stdout-stderr/
+# http://stackoverflow.com/questions/22211658/implementing-my-own-event-loop-in-a-wxpython-application
+# http://wiki.wxpython.org/LongRunningTasks
+# http://wiki.wxpython.org/Non-Blocking%20Gui
+#####################################################################
+class _Printer(object):
+    def __init__(self,aWxTextCtrl):
+        self.out = aWxTextCtrl
+
+    def write(self,line):
+        self.out.WriteText(line)
+
+    def flush(self):    
+        """force flushing/updating gui"""
+        # make the size small enough to visually hide the splash
+        # the timer interval cannot be too small, >=50 ms
+        class _ModifiedTimedMessageDialog(wx.Dialog):
+            def __init__(self, message, title, ttl=10):
+                wx.Dialog.__init__(self, None, -1, title,size=(0, 0),style=wx.SYSTEM_MENU)
+                self.CenterOnScreen(wx.BOTH)
+                self.timeToLive = ttl
+                self.timer = wx.Timer(self)
+                self.timer.Start(100) #Generate a timer event every second
+                self.Bind(wx.EVT_TIMER, self.onTimer, self.timer)
+            def onTimer(self, evt):
+                self.timeToLive -= 2
+                if self.timeToLive <= 0:
+                    self.timer.Stop()
+                    self.Close()
+        dlg = _ModifiedTimedMessageDialog('', 'flushing', 0)               
+        dlg.ShowModal()
+
+class _XPrinter(object):
+    def __init__(self, printer):
+        self.terminal = sys.__stdout__
+        self.printer = printer
+
+    def write(self, message):
+        self.terminal.write(message)
+        self.printer.write(message)
+        self.printer.flush()
+
+    def flush(self):
+        self.terminal.flush()
+        self.printer.flush()
+
+class XPrinter(wx.Frame):
+    """
+    Display a window to capture print output
+    if on, both terminal and window (updating gui will greatly increase script execution time)
+    if off, only terminal
+
+    Methods: on/off
+
+    Examples:
+        xprinter = XPrinter()
+        xprinter.on()
+        print 'will be shown on window'
+        xprinter.off()
+        print 'will be shown in terminal'
+        xprinter.on()
+        print 'on window again'
+
+        for x in range(100):
+            print "I am a line of " + str(x)
+            # time.sleep(0.01)   
+    """
+    def __init__(self):
+        wx.Frame.__init__(self, None, wx.ID_ANY, "Welcome to this App!", size=(700,300))
+    
+        # GUI stuff
+        # Add a panel so it looks the correct on all platforms
+        thePanel = wx.Panel(self, wx.ID_ANY)
+        txtCtrl = wx.TextCtrl(thePanel, wx.ID_ANY, size=(300,100),
+                          style = wx.TE_MULTILINE|wx.TE_READONLY|wx.HSCROLL)
+        txtCtrl.Enable(False)
+        self.theBtn = wx.Button(thePanel, wx.ID_ANY, 'Close')
+        # Add widgets to a sizer        
+        theSizer = wx.BoxSizer(wx.VERTICAL)
+        theSizer.Add(txtCtrl, 1, wx.ALL|wx.EXPAND, 5)
+        theSizer.Add(self.theBtn, 0, wx.ALL|wx.CENTER, 5)
+        thePanel.SetSizer(theSizer)
+        self.Layout()
+        self.Center()
+        self.Show()
+        self.theBtn.SetDefault()
+        self.theBtn.SetFocus()
+        self.ToggleWindowStyle(wx.STAY_ON_TOP)
+
+        # printer
+        self._printer = _Printer(txtCtrl)
+        self._xprinter = _XPrinter(self._printer)
+
+        # Event Handlers
+        self.Bind(wx.EVT_BUTTON, self.OnClose, self.theBtn)
+        self.Bind(wx.EVT_CHAR_HOOK, self.OnKeyUP)
+
+    # escape key to quit
+    def OnKeyUP(self, evt):
+        keyCode = evt.GetKeyCode()
+        if keyCode == wx.WXK_ESCAPE:
+            self.OnClose(evt)
+
+    def OnClose(self,evt):
+        self.Close()    
+
+    def on(self):
+        sys.stdout = sys.__stdout__
+        print "+++++Starting displaying in GUI+++++\n"
+        sys.stdout = self._xprinter
+
+    def off(self):
+        self._xprinter.flush()
+        sys.stdout = sys.__stdout__
+        print "+++++Stopping displaying in GUI+++++\n"
+
 if __name__ == "__main__":
     app = wx.App(redirect=False)
-    import os
-    items = [('ID:', ''),
-        ('ID:', 'uni30122133231231123235'),
-        ('ID:', 1001),
-        ('IDs:', [1001, 1002]),
-        ('Logical Switch:', 'Checked?', False),
-        ('Gender:', ['Female', 'Male'], 0),
-        ('Race:', ['Black', 'White', 'Other'], -1),
-        (''),
-        ({'Selecte Input Directory...': "GetDir()"},''),
-        ({'Selecte Output Directory...': "GetDir()"},''),
-        ({'Save as...': "SetFile()"},''),
-        ({'Selecte Files...': "GetFile(multiple=True)"},[]),
-        ('Majors:\n(Can select more than one)',('Psychology','Math','Biology'), 0),
-        ({"Output File Name(*.csv):": "SetFile(directory='%s', filename='output.csv', wildcard='CSV Files (*.csv)|*.csv')" % os.getcwd()}, '')]
-
-    values = Inputs(items=items, instruction=__doc__)
-    print values
+    import os, time
+    xprinter = XPrinter()
+    xprinter.on()
+    print 'will be shown on window'
+    xprinter.off()
+    print 'will be shown in terminal'
+    xprinter.on()
+    print 'on window again'*30
 
+    time.sleep(5)
+
+    # xprinter.off()
+    for x in range(30):
+        print "I am a line of " + str(x)
+        # time.sleep(0.01)
+    
+    # items = [('ID:', ''),
+    #     ('ID:', 'uni30122133231231123235'),
+    #     ('ID:', 1001),
+    #     ('IDs:', [1001, 1002]),
+    #     ('Logical Switch:', 'Checked?', False),
+    #     ('Gender:', ['Female', 'Male'], 0),
+    #     ('Race:', ['Black', 'White', 'Other'], -1),
+    #     (''),
+    #     ({'Selecte Input Directory...': "GetDir()"},''),
+    #     ({'Selecte Output Directory...': "GetDir()"},''),
+    #     ({'Save as...': "SetFile()"},''),
+    #     ({'Selecte Files...': "GetFile(multiple=True)"},[]),
+    #     ('Majors:\n(Can select more than one)',('Psychology','Math','Biology'), 0),
+    #     ({"Output File Name(*.csv):": "SetFile(directory='%s', filename='output.csv', wildcard='CSV Files (*.csv)|*.csv')" % os.getcwd()}, '')]
+    
+    # values = Inputs(items=items,width=1000,instruction=__doc__)
+    # print values
+
+    # put at the bottom to keep gui window alive
+    # app.MainLoop()
 
 
